#include <ros/ros.h>

#include <sensor_msgs/LaserScan.h>
#include <tf/transform_broadcaster.h>
#include <nav_msgs/Odometry.h>

#include <iostream>
#include <iomanip>
#include <fstream>
#include <string.h>
using namespace std;

int main(int argc, char** argv){
  ifstream dfile;
  //dfile.open("../Scout_log__20_12_2009_03_47_36_338587.dat");
  dfile.open("../Log2.dat");
  if (!dfile.is_open()) {
    cout<<"ERROR: File is not opened"<<endl;
    return(-1);
  }

  ros::init(argc, argv, "laser_scan_publisher");

  ros::NodeHandle n;
  ros::Publisher scan_pub = n.advertise<sensor_msgs::LaserScan>("/base_scan", 50);
  ros::Publisher odom_pub = n.advertise<nav_msgs::Odometry>("odom", 50);
  static tf::TransformBroadcaster br;

  tf::Transform transform;
  ros::Time this_time;

  unsigned int num_readings = 100;
  double laser_frequency = 40;
  double ranges[num_readings];
  double intensities[num_readings];

  // Store last pose to skip over long pauses... these make me bo-red.
  double old_x = 999.0;
  double old_y = 998.0;
  double old_th = 887.0;
  bool new_data;

  double data[689];

  int count = 0;
  ros::Rate loop_rate(75);

  while(n.ok()&&dfile.good()) {

    new_data = false;

    while( !new_data ) {
      string line;
      getline (dfile,line);
      string::size_type index;
      index = line.find(" ");
      int cnt=0;

      while (index != string::npos) {
	string number;
	number.assign(line.begin(), line.begin()+index);
	line.erase(0, index + 1);
	data[cnt++]=atof(number.c_str());
	index = line.find(" ");
      }

      if( data[1] != old_x || data[2] != old_y || data[3] != old_th ) {
	new_data = true;
	old_x = data[1];
	old_y = data[2];
	old_th = data[3];
      }
    }

    double intpart;
    double nsecs=1000000000*modf(data[0],&intpart);
    ros::Time scan_time((int)intpart,(int)nsecs);
    cout<<setprecision (15) <<"time "<<data[0]<<" "<<intpart<<" "<<nsecs<<" "<<scan_time<<" Pose ["<<data[1]<<","<<data[2]<<","<<data[3]<<"]"<<endl;

    // Construct the transform:

    this_time = ros::Time::now();

    transform.setOrigin( tf::Vector3(data[1], data[2], 0.0) );
    transform.setRotation( tf::Quaternion(data[3], 0, 0) );

    //populate the LaserScan message
    sensor_msgs::LaserScan scan;
    scan.header.stamp = this_time;
    scan.header.frame_id = "/base_link";
    scan.angle_min = data[5];
    scan.angle_max = data[6];
    scan.angle_increment = (scan.angle_max-scan.angle_min) / data[4];
    scan.time_increment = (1 / laser_frequency) / (num_readings);
    scan.range_min = 2.0;
    scan.range_max = 4.990;

    scan.set_ranges_size(data[4]);
    scan.set_intensities_size(data[4]);
    unsigned int s=data[4]+7;
    for(unsigned int i = 0; i <data[4] ; i++){
      scan.ranges[i] = data[8+i];
      scan.intensities[i] = 100;
    }

    scan_pub.publish(scan);
    br.sendTransform(tf::StampedTransform(transform, this_time, "/odom", "/base_link"));

    geometry_msgs::Quaternion odom_quat=tf::createQuaternionMsgFromYaw(data[3]);
    nav_msgs::Odometry odom;
    odom.header.stamp = this_time;
    odom.header.frame_id = "odom";
    
    //set the position
    odom.pose.pose.position.x = data[1];
    odom.pose.pose.position.y = data[2];
    odom.pose.pose.position.z = 0.0;
    odom.pose.pose.orientation = odom_quat;
    //publish the message
    odom_pub.publish(odom);

    ++count;
    loop_rate.sleep();

  }
}
